(module hexxagon mzscheme
  (require (lib "mred.ss" "mred")
	   (lib "class.ss")
           (lib "unit.ss")
	   (lib "list.ss"))
  
  (define basepath 0)
  
  (let ((x 0) (y 0) (z 0))
    (set!-values (x y z) (split-path (find-system-path 'exec-file )))
    (set! basepath (build-path x 'up "lib" "hexxagon" "include")))
  
  (define splash-frame (new frame% (label "Splash Screen") (width 369) (height 229) (style '(float no-caption))))
  (define splash=bmp (make-object bitmap% (build-path basepath "images" "splash.xpm") 'xpm))
  (send splash-frame show #t)
  (send
   (send 
    (instantiate canvas% (splash-frame '(transparent) (lambda (x y) (send y clear) (send y draw-bitmap splash=bmp 0 0))))
    get-dc)
   draw-bitmap splash=bmp 0 0)
  (send splash-frame show #t)
  
  
  
  ;                                                                                          
  ;                                                                                          
  ;                                                                                          
  ;                                                                                          
  ;  ;;                                   ;;                    ;;                           
  ;   ;                                    ;                     ;                           
  ;   ; ;;;    ;;;;    ;;;;   ;; ;;;   ;;; ;           ;;; ;     ;     ;;;;    ;;;;;   ;;;;; 
  ;   ;;   ;  ;    ;  ;    ;   ;;     ;   ;;          ;   ;;     ;    ;    ;  ;    ;  ;    ; 
  ;   ;    ;  ;    ;   ;;;;;   ;      ;    ;          ;          ;     ;;;;;   ;;;;    ;;;;  
  ;   ;    ;  ;    ;  ;    ;   ;      ;    ;          ;          ;    ;    ;       ;       ; 
  ;   ;;   ;  ;    ;  ;   ;;   ;      ;   ;;          ;    ;     ;    ;   ;;  ;    ;  ;    ; 
  ;  ;; ;;;    ;;;;    ;;; ;; ;;;;;    ;;; ;;          ;;;;    ;;;;;   ;;; ;; ;;;;;   ;;;;;  
  ;                                                                                          
  ;                                                                                          
  ;                                                                                          
  ;                                                                                          
  
  
  (define (make-board no_rows)
    ;;;;;;;;;;;;;;;;;;;;;;DISPATCH;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (dispatch msg)
      (cond    
        ((eq? msg 'get-val) get-val)
        ((eq? msg 'set-val) set-val)
        ((eq? msg 'first-ring) 
         (lambda (r-theta) 
           (car (vector-ref (vector-ref move_board (car r-theta)) (cdr r-theta)))))
        ((eq? msg 'second-ring) 
         (lambda (r-theta) 
           (cdr (vector-ref (vector-ref move_board (car r-theta)) (cdr r-theta)))))
        ((eq? msg 'get-total) (get-total))
        ((eq? msg 'board) board)
        ((eq? msg 'replace-board) replace-board)
        ((eq? msg 'get-no-cells) no_of_cells)
        ((eq? msg 'listp) 
         (lambda (x) (if (= x 1) listp1 listp-1)))
        (else "wrong msg to make-board")))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;INIT_BOARD;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (init_board) 
      (define (init_board_helper no_of_rows)
        (define (make_list n list)
          (if (= n 0) list
              (make_list (- n 1) (cons (- n 1) list))))
        (map (lambda (x) 
               (cond 
                 ((= x 0) (make-vector 1 0))
                 ((> 3 (- no_of_rows x)) (make-vector (* x 6) ()))
                 (else (make-vector (* x 6) 0 ))))
             (make_list no_of_rows ())))
      (list->vector (init_board_helper (+ no_rows 2))))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;GET-VAL;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (get-val r-theta)
      (vector-ref (vector-ref board (car r-theta)) (cdr r-theta)))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;SET-VAL;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (set-val r-theta new_val)
      (let ((value (get-val r-theta)))
        (cond
          ((eq? value 1) (set! listp1 (remove r-theta listp1)))
          ((eq? value -1) (set! listp-1 (remove r-theta listp-1)))))
      (cond
        ((eq? new_val 1) (set! listp1 (cons r-theta listp1)))
        ((eq? new_val -1) (set! listp-1 (cons r-theta listp-1)))
        ((eq? new_val ()) (set! no_of_cells (- no_of_cells 1))))
      (vector-set! (vector-ref board (car r-theta)) (cdr r-theta) new_val))
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;GET-TOTAL;;;;;;;;;;;;;;;;;;;;;;;
    (define (get-total)
      (define goto_length (- (vector-length board) 3)) 
      (define (outer_counter r)
        (define current_vector (vector-ref board r))  
        (define current_length (vector-length current_vector))
        (define (inner_counter theta)
          (if (= theta current_length) 0
              (+ (let ((temp (vector-ref current_vector theta))) 
                   (if (eq? temp ()) 0 temp)) (inner_counter (+ theta 1)))))
        (if (= r goto_length) (inner_counter 0)
            (+ (inner_counter 0) (outer_counter (+ r 1)))))
      (outer_counter 0))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;UPDATE-MOVE_BOARD;;;;;;;;;;;;;;;;;;;;
    (define (update-move_board)
      
      (define (get-second_ring r theta)
        (define returnval (make-vector 12))
        (define firstring (get-adjecent r theta))
        (define (helper k)
          (if (= k 6) returnval
              (begin 
                (let*
                    ((adjecents (vector-ref firstring k)) 
                     (entrys (get-adjecent2 (car adjecents) (cdr adjecents) k (modulo (+ k 1) 6))))
                  (begin
                    (vector-set! returnval (* 2 k) (car entrys))
                    (vector-set! returnval (+ (* 2 k) 1) (cdr entrys))))
                (helper (+ k 1)))))
        (helper 0))
      
      (define (get-adjecent2 r theta dir1 dir2)
        (let
            ((val (get-adjecent r theta)))
          (cons (vector-ref val dir1) (vector-ref val dir2))))
      
      (define (get-adjecent r theta)
        (define (rem-mod d1 d2)
          (if (= d2 0) d1
              (remainder d1 d2)))
        (if (= r 0) 
            (vector '(1 . 0) '(1 . 1) '(1 . 2) '(1 . 3) '(1 . 4) '(1 . 5))
            (letrec
                ((rem (rem-mod theta r))
                 (ratio (floor (/ theta r)))
                 (thetabig (+ rem (* (+ r 1) ratio)))
                 (thetasmall (rem-mod (+ rem (* (- r 1) ratio)) (* 6 (- r 1))))
                 (vector_unnumbered (if (= rem 0) 
                                        (vector 
                                         (cons (+ r 1) thetabig)
                                         (cons (+ r 1) (rem-mod (+ thetabig 1) (* 6 (+ r 1))))
                                         (cons r (rem-mod (+ theta 1) (* 6 r)))
                                         (cons (- r 1) thetasmall)
                                         (cons r (modulo (- theta 1) (* 6 r)))
                                         (cons (+ r 1) (modulo (- thetabig 1) (* 6 (+ r 1)))))
                                        (vector 
                                         (cons (+ r 1) thetabig)
                                         (cons (+ r 1) (rem-mod (+ thetabig 1) (* 6 (+ r 1))))
                                         (cons r (rem-mod (+ theta 1) (* 6 r)))
                                         (cons (- r 1) thetasmall)
                                         (cons (- r 1) (modulo (- thetasmall 1) (* 6 (- r 1))))
                                         (cons r (modulo (- theta 1) (* 6 r))))))
                 (vector_numbered (make-vector 6))
                 (arrange (lambda (pointer counter)
                            (cond 
                              ((= counter 6) vector_numbered)
                              ((= pointer 5) (begin
                                               (vector-set! vector_numbered pointer (vector-ref vector_unnumbered counter))
                                               (arrange 0 (+ counter 1))))
                              (else (begin
                                      (vector-set! vector_numbered pointer (vector-ref vector_unnumbered counter))
                                      (arrange (+ pointer 1) (+ counter 1)))))))) 
              (arrange ratio 0))))
      
      (define outer_length  (- (vector-length move_board) 2))
      
      
      (define (outer-iter r)
        (define inner_vector (vector-ref move_board r))
        (define inner_length (vector-length inner_vector))
        
        (define (inner-iter theta)
          
          (define (for-each-cell)
            (cons 
             (get-adjecent r theta) 
             (get-second_ring r theta)))
          
          (if (= theta inner_length) void
              (begin
                (vector-set! (vector-ref move_board r) theta (for-each-cell))
                (inner-iter (+ theta 1)))))
        
        (if (= r outer_length) void
            (begin
              (inner-iter 0)
              (outer-iter (+ r 1)))))
      
      (outer-iter 0))             ;;;update-move_board;;;;
    
    ;;;;;;;;;;;;;replace-board;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (replace-board new_board)    
      (define (outer-iter r)
        (define (inner-iter theta)
          (let ((val (get-val (cons r theta))))
            (cond
              ((eq? val 1) (set! listp1 (cons (cons r theta) listp1)))
              ((eq? val -1) (set! listp-1 (cons (cons r theta) listp-1)))
              ((eq? val ()) (set! no_of_cells (- no_of_cells 1)))))
          (if (> theta 0) (inner-iter (- theta 1)) void))
        
        (inner-iter (- (vector-length (vector-ref move_board r)) 1))
        
        (if (> r 0) (outer-iter (- r 1)) void))
      
      (set! board (copy-board new_board))
      (set! no_of_cells (+ 1 (* 3 no_rows (- no_rows 1))))
      (set! listp1 ())
      (set! listp-1 ())
      (outer-iter (- (vector-length move_board) 3)))
    
    
    ;;;;;;;;;;;;;;;;
    (define no_of_cells (+ 1 (* 3 no_rows (- no_rows 1)))) 
    (define board (init_board))
    (define move_board (init_board))
    (define listp1 ())
    (define listp-1 ())
    (update-move_board)
    
    dispatch)
  
  
  
  ;                                                                 
  ;                                                                 
  ;                                                                 
  ;          +++                                                    
  ;            $                                                    
  ;            $                                                    
  ;  +$*+#+    $    +++#+ +$+ :#+       +$++++  -+++- +$+ :#+ -++++ 
  ;   $+ *$    $    +   $  #*  +         $:$:$  #: :#  #*  +  #-  # 
  ;   $   $    $     +++$  ++ -+         $ $ $  $   $  ++ -+ :$+++$*
  ;   $   $    $    #+  $   $ +          $ $ $  $   $   $ +  -$     
  ;   $  +#    $    $  +$   +++          $ $ $  #* *#   +++   ++    
  ;   $#++   ++$++  +#++$+  :$:         *$ $-$* -+++-   :$*    +#++ 
  ;   $                      +                                      
  ;  +$+*                  *+#-                                     
  ;                                                         @@@@@@@@
  
  (define (play-move in_cor fi_cor)
    (define play_val ((myboard 'get-val) in_cor))
    
    (define (check-presence pos-cordinate list-vector)
      (define (counter n)
        (if (= n (vector-length list-vector))
            #f
            (if (equal? pos-cordinate (vector-ref list-vector n))
                #t
                (counter (+ n 1)))))
      (counter 0))
    
    (define (play-adjescent)
      (define adjescent-vector ((myboard 'first-ring) fi_cor))
      (define list-2-return ())
      
      (define (counter n)
        (if (= n (vector-length adjescent-vector)) list-2-return
            (let*
                ((pos_data (vector-ref adjescent-vector n))
                 (pos_val ((myboard 'get-val) pos_data)))
              (cond
                ((eq? pos_val ()) (counter (+ n 1)))
                ((= (+ play_val pos_val) 0)
                 (begin
                   ((myboard 'set-val) pos_data play_val)
                   (set! list-2-return (cons pos_data list-2-return))
                   (counter (+ n 1))))
                (else (counter (+ n 1)))))))
      (list->vector (counter 0))) 
    
    (cond
      ((not (eq? ((myboard 'get-val) fi_cor) 0)) 'error)
      ((check-presence fi_cor ((myboard 'first-ring) in_cor))
       (begin
         ((myboard 'set-val) fi_cor play_val)
         (cons 'first (play-adjescent))))
      ((check-presence fi_cor ((myboard 'second-ring) in_cor))
       (begin
         ((myboard 'set-val) fi_cor play_val)
         ((myboard 'set-val) in_cor 0)
         (cons 'second (play-adjescent))))
      (else 'error)))
  
  
  
  ;                                                                                  
  ;                                                                                  
  ;                                                                                  
  ;                                                                                  
  ;                                          ;;                                   ;; 
  ;                                           ;                                    ; 
  ;    ;;; ;   ;;;;   ;; ;;   ;;; ;;;         ; ;;;    ;;;;    ;;;;   ;; ;;;   ;;; ; 
  ;   ;   ;;  ;    ;   ;;  ;   ;   ;          ;;   ;  ;    ;  ;    ;   ;;     ;   ;; 
  ;   ;       ;    ;   ;   ;   ;   ;  ;;;;;;  ;    ;  ;    ;   ;;;;;   ;      ;    ; 
  ;   ;       ;    ;   ;   ;    ; ;           ;    ;  ;    ;  ;    ;   ;      ;    ; 
  ;   ;    ;  ;    ;   ;   ;    ; ;           ;;   ;  ;    ;  ;   ;;   ;      ;   ;; 
  ;    ;;;;    ;;;;    ;;;;      ;           ;; ;;;    ;;;;    ;;; ;; ;;;;;    ;;; ;;
  ;                    ;         ;                                                   
  ;                   ;;;      ;;;                                                   
  ;                                                                                  
  ;                                                                                  
  
  
  (define (copy-board changed_board)
    (vector 
     (list->vector (vector->list (vector-ref changed_board 0)))
     (list->vector (vector->list (vector-ref changed_board 1)))
     (list->vector (vector->list (vector-ref changed_board 2)))
     (list->vector (vector->list (vector-ref changed_board 3)))
     (list->vector (vector->list (vector-ref changed_board 4)))
     (list->vector (vector->list (vector-ref changed_board 5)))
     (list->vector (vector->list (vector-ref changed_board 6)))))
  
  
  
  
  ;                                                                                  
  ;                                                                                  
  ;                                                                                  
  ;                                                                                  
  ;                                             ;;                       ;           
  ;                                              ;                                   
  ;    ;;; ;   ;;;;  ;; ;  ;  ;; ;;              ;     ;;;;    ;;; ;;  ;;;     ;;; ; 
  ;   ;   ;;  ;    ;  ;; ;; ;  ;;  ;             ;    ;    ;  ;   ;;     ;    ;   ;; 
  ;   ;       ;    ;  ;  ;  ;  ;   ;  ;;;;;;     ;    ;    ;  ;    ;     ;    ;      
  ;   ;       ;    ;  ;  ;  ;  ;   ;             ;    ;    ;  ;    ;     ;    ;      
  ;   ;    ;  ;    ;  ;  ;  ;  ;   ;             ;    ;    ;  ;   ;;     ;    ;    ; 
  ;    ;;;;    ;;;;  ;;; ;; ;  ;;;;            ;;;;;   ;;;;    ;;; ;   ;;;;;   ;;;;  
  ;                            ;                                   ;                 
  ;                           ;;;                              ;;;;                  
  ;                                                                                  
  ;                                                                                  
  
  
  
  (define (comp-logic)
    
    (define init_val_board (myboard 'get-total))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;reverse;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
    (define (reverse in_cor fi_cor memory)
      (define val ((myboard 'get-val) fi_cor))
      (define (reverse-helper n)
        ((myboard 'set-val) (vector-ref (cdr memory) n) (- val))
        (if (= n 0) void (reverse-helper (- n 1))))
      
      ((myboard 'set-val) in_cor val)
      ((myboard 'set-val) fi_cor 0)
      (let ((mem_length (vector-length (cdr memory))))
        (if (= mem_length 0) void
            (reverse-helper (- mem_length 1)))))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;checkwin;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (checkwin player_id)
      (define full? (+ (length ((myboard' listp) 1)) (length ((myboard' listp) -1))))
      (if (= full? (myboard 'get-no-cells))
          (if (> (myboard 'get-total) (/ (myboard 'get-no-cells) 2))
              (* infinity player_id) 
              (* neg_infinity player_id))
          (* neg_infinity player_id)))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;alpha beta;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (alphabeta depth alpha beta player_id)
      (define pos ())
      (define path())
      (define playedring1list ())
      (define temp ())                               ;;used for rough work
      
      (define played? #f)
      (define working #t)
      
      (define (loop listp) 
        (define point2play (car listp))
        (define ring1 ((myboard 'first-ring) point2play))
        (define ring2 ((myboard 'second-ring) point2play))
        
        
        (define (playtoring2 counter)
          (set! temp (vector-ref ring2 counter))
          (if (eq? ((myboard 'get-val) temp) 0)
              (let*
                  ((reverse_data (play-move point2play temp))
                   (tttt (set! played? #t))
                   (value (alphabeta (- depth 1) (- beta) (- alpha) (- player_id)))
                   (recur_val (car value)) 
                   (ttt (if (> recur_val alpha)
                            (begin
                              (set! pos (cons point2play temp))
                              (set! path (cdr value))
                              (set! alpha recur_val)
                              (if (>= alpha beta)
                                  (set! working #f)
                                  void)
                              )
                            void)))
                (reverse point2play temp reverse_data))
              void)
          (if (and (> counter 0) working) 
              (playtoring2 (- counter 1))
              void))
        
        
        (define (playtoring1 counter)
          (define (present? item l)
            (if (null? l) #f
                (if (eq? (car l) item) #t
                    (present? item (cdr l)))))
          
          (set! temp (vector-ref ring1 counter))
          
          (if (eq? ((myboard 'get-val) temp) 0)
              (if (present? temp playedring1list) void
                  (let* 
                      ((reverse_data (play-move point2play temp))
                       (tttt (set! played? #t))
                       (value (alphabeta (- depth 1) (- beta) (- alpha) (- player_id)))
                       (recur_val (car value)) 
                       (ttt (if (> recur_val alpha)
                                (begin
                                  (set! pos (cons point2play temp))
                                  (set! path (cdr value))
                                  (set! alpha recur_val)
                                  (if (>= alpha beta)
                                      (set! working #f)
                                      void)
                                  )
                                void)))
                    (set! playedring1list (cons temp playedring1list))
                    (reverse point2play temp reverse_data)))
              void)
          (if (and (> counter 0) working) 
              (playtoring1 (- counter 1))
              void))
        
        (cond
          ((not working) (list (- alpha) pos path))
          (else 
           (playtoring1 5)
           (playtoring2 11)))
        
        (if (null? (cdr listp))
            (if played?
                (list (- alpha) pos path)
                (let 
                    ((won? (checkwin player_id)))
                  (list won? pos path)))
            (loop (cdr listp))))
      
      
      (if (= depth 0)
          (let*
              ((x ((if (even? ply) + -) (- init_val_board (myboard 'get-total)))))
            (cons x ()))
          (let ((x ((myboard 'listp) player_id)))
            (if (null? x)
                (cons (* neg_infinity player_id) ())
                (loop x)))))
    
    (alphabeta ply (- infinity) infinity 1))
  
  ;;;;;;;;;;;;;;;;;end of complogic;;;;;;;;;;;;
  
  ;;;;;;;;;;;;;;;;;;;;checkwin;;;;;;;;;;;;;;
  (define (checkwin player_id)
    (define l1 ((myboard 'listp) player_id))
    (define l2 ((myboard 'listp) (- player_id)))
    
    (if (= (+ (length l1) (length l2)) (myboard 'get-no-cells))
        (let ((x (myboard 'get-total)))
          (cond
            ((= x 0) 0)
            ((> x 0) 1)
            ((< x 0) -1)))
        (letrec
            ((checkzero (lambda (l)
                          (if (null? l) #f
                              (if (eq? ((myboard 'get-val) (car l)) 0) #t
                                  (checkzero (cdr l))))))
             (checklist (lambda (l)
                          (cond
                            ((null? l) #f)
                            ((eq? (checkzero (vector->list ((myboard 'second-ring) (car l)))) #t) #t)
                            ((eq? (checkzero (vector->list ((myboard 'first-ring) (car l)))) #t) #t)
                            (else (checklist (cdr l)))))))
          (if (checklist l2) 2 player_id))))
  ;;;;;;;;;;;;;;end of checkwin;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  
  ;                                                                          
  ;                                                                          
  ;                                                                          
  ;                                                                          
  ;                                                     ;;                   
  ;                                                      ;                   
  ;    ;;; ;   ;;;;  ;; ;  ;  ;; ;;           ;; ;;      ;     ;;;;   ;;; ;;;
  ;   ;   ;;  ;    ;  ;; ;; ;  ;;  ;           ;;  ;     ;    ;    ;   ;   ; 
  ;   ;       ;    ;  ;  ;  ;  ;   ;           ;   ;     ;     ;;;;;   ;   ; 
  ;   ;       ;    ;  ;  ;  ;  ;   ;           ;   ;     ;    ;    ;    ; ;  
  ;   ;    ;  ;    ;  ;  ;  ;  ;   ;           ;   ;     ;    ;   ;;    ; ;  
  ;    ;;;;    ;;;;  ;;; ;; ;  ;;;;            ;;;;    ;;;;;   ;;; ;;    ;   
  ;                            ;               ;                         ;   
  ;                           ;;;             ;;;                      ;;;   
  ;                                                                          
  ;                                                                          
  
  (define (comp-play)
    (define curmax ())
    
    (show-working #t)
    (with-handlers ((exn:fail? (lambda (exn) void)))
      (let*
          ((result (comp-logic))
           (ttt (set! curmax (car result)))
           (play_c1 (caaadr result))
           (play_c2 (cdaadr result))
           (play_to (cdadr result)))
        (play-move (cons play_c1 play_c2) play_to)))
    
    (let ((x (checkwin 1)))
      (if (= x 2) void (show-game-end x)))        
    
    (show-working #f)
    (draw myboard)) 
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;new game;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  (define (new-game)
    ((myboard 'replace-board) replace_board)
    (draw myboard)
    (if (eq? first_player 'comp) (comp-play) void))
  
  
  ;                                                                                                                                  
  ;                                                                                                                                  
  ;                                                                                                                                  
  ;                                                                                                                                  
  ;                      ;                                                                              ;;       ;            ;;     
  ;                                                                                                      ;                     ;     
  ;  ;; ;  ;   ;;;;    ;;;    ;; ;;          ;; ;  ;   ;;;;   ;;  ;;   ;;;;;   ;;;;            ;;; ;     ;     ;;;     ;;; ;   ; ;;;;
  ;   ;; ;; ; ;    ;     ;     ;;  ;          ;; ;; ; ;    ;   ;   ;  ;    ;  ;    ;          ;   ;;     ;       ;    ;   ;;   ;  ;  
  ;   ;  ;  ;  ;;;;;     ;     ;   ;          ;  ;  ; ;    ;   ;   ;   ;;;;   ;;;;;;          ;          ;       ;    ;        ;;;   
  ;   ;  ;  ; ;    ;     ;     ;   ;          ;  ;  ; ;    ;   ;   ;       ;  ;               ;          ;       ;    ;        ; ;   
  ;   ;  ;  ; ;   ;;     ;     ;   ;          ;  ;  ; ;    ;   ;  ;;  ;    ;  ;               ;    ;     ;       ;    ;    ;   ;  ;  
  ;  ;;; ;; ;  ;;; ;;  ;;;;;  ;;; ;;;        ;;; ;; ;  ;;;;     ;; ;; ;;;;;    ;;;;;           ;;;;    ;;;;;   ;;;;;   ;;;;   ;;  ;;;
  ;                                                                                                                                  
  ;                                                                                                                                  
  ;                                                                                                                                  
  ;                                                                                                                                  
  
  (define (mouseclick xy)
    (define r-theta (xy->rtheta xy))
    (define (lift-ball)
      (drawpoint (car r-theta) (cdr r-theta) -1 -1)
      (drawlist ((myboard 'first-ring) r-theta) 1)
      (drawlist ((myboard 'second-ring) r-theta) 2)
      (send canvas-dc draw-bitmap (send dc get-bitmap) 0 0)
      (set! is_lifted? #t)
      (set! cor_lifted r-theta))
    
    (define (drop-ball)
      (if is_lifted? (draw myboard) void)
      (set! is_lifted? #f))
    
    (if (eq? r-theta 'out-of-board) (drop-ball)
        (if is_lifted?
            (let ((error (play-move cor_lifted r-theta)))
              (drop-ball)
              (cond
                ((eq? error 'error) void)
                (else 
                 (let ((x (checkwin -1)))
                   (if (= x 2) (comp-play) (show-game-end x))))))
            (if (eq? ((myboard 'get-val) r-theta) user)
                (lift-ball)
                void))))
  
  
  
  
  
  ;                                                                                          
  ;                                                                                          
  ;                                                                                          
  ;                                                                                          
  ;                   ;;                                                                     
  ;            ;       ;               ;                   ;          ;                      
  ;   ;; ;;;  ;;;;;    ; ;;    ;;;;   ;;;;;    ;;;;      ;;            ;;     ;;  ;;  ;;; ;;;
  ;    ;;      ;       ;;  ;  ;    ;   ;      ;    ;    ;                ;     ;  ;    ;   ; 
  ;    ;       ;       ;   ;  ;;;;;;   ;       ;;;;;  ;;      ;;;;;;      ;;    ;;     ;   ; 
  ;    ;       ;       ;   ;  ;        ;      ;    ;    ;                ;      ;;      ; ;  
  ;    ;       ;   ;   ;   ;  ;        ;   ;  ;   ;;     ;;            ;;      ;  ;     ; ;  
  ;   ;;;;;     ;;;   ;;; ;;;  ;;;;;    ;;;    ;;; ;;      ;          ;       ;;  ;;     ;   
  ;                                                                                      ;   
  ;                                                                                    ;;;   
  ;                                                                                          
  ;                                                                                          
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;rtheta->xy;;;;;;;;;;;;;;;;;;;
  
  (define (rtheta->xy r theta)
    
    (if (= r 0) (cons centrex centrey)
        (let* 
            ((ratio (floor (/ theta r)))
             (dist (* 2.4 d))
             (ang (if (= r 0) 2 (* pi (/ ratio 3))))
             (ang2 (if (= r 0) 2 (* pi (/ (+ ratio 1) 3))))
             (y1 (+ centrey (* r (cos ang) dist)))
             (x1 (- centrex (* r (sin ang) dist)))
             (y2 (+ centrey (* r (cos ang2) dist)))
             (x2 (- centrex (* r (sin ang2) dist)))
             (x (+ x1 (/ (* (remainder theta r) (- x2 x1)) r)))
             (y (+ y1 (/ (* (remainder theta r) (- y2 y1)) r))))
          (cons x y))))
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;xy->rtheta;;;;;;;;;;;;;;;;;;;;;;;;;;
  (define (xy->rtheta xy)
    (define x (car xy))
    (define y (cdr xy))
    
    (define (eq1 x y)
      (+ y (* x (sqrt 3))))
    (define (eq2 x y)
      (- y (* x (sqrt 3))))
    
    (define (rcounter r)
      
      (define (thetacounter theta)
        (define x0y0 (rtheta->xy r theta))
        (define x0 (car x0y0))
        (define y0 (cdr x0y0))
        
        (if (and
             (> (+ (- y y0) (* 1.2 d)) 0)
             (< (- (- y y0) (* 1.2 d)) 0)
             (> (+ (- (eq1 x y) (eq1 x0 y0)) (* 2.4 d)) 0)
             (< (- (- (eq1 x y) (eq1 x0 y0)) (* 2.4 d)) 0)
             (> (+ (- (eq2 x y) (eq2 x0 y0)) (* 2.4 d)) 0)
             (< (- (- (eq2 x y) (eq2 x0 y0)) (* 2.4 d)) 0))
            (cons r theta)
            (if (< theta (- (* 6 r) 1))
                (thetacounter (+ theta 1))
                (rcounter (+ r 1)))))
      
      (if (< r boardsize) (thetacounter 0) 'out-of-board))
    
    (rcounter 0))
  
  
  
  ;                                                                 
  ;                                                                 
  ;                                                                 
  ;                                                                 
  ;                                :                    :           
  ;                                $                    $           
  ;    ++++  -+++- +$*+#+  +++++ ++$+++  +++#+ +$*+#+ ++$+++  +++++ 
  ;   #+  +  #: :#  $+ :#  $   +   $     +   $  $+ :#   $     $   + 
  ;   $      $   $  $   $  +#+*    $      +++$  $   $   $     +#+*  
  ;   $      $   $  $   $    +++   $     #+  $  $   $   $       +++ 
  ;   #+     #* *#  $   $  +   $   $     $  +$  $   $   $     +   $ 
  ;    +#++  -+++- +$+ +$+ +++++   +#++  +#++$++$+ +$+  +#++  +++++ 
  ;                                                                 
  ;                                                                 
  ;                                                         +#@  @@@
  
  (define pi 3.14159)
  (define infinity 999999)
  (define neg_infinity (- infinity))
  
  (define ply 3)
  
  (define ring1_length 6)
  (define ring2_length 12) 
  (define d 25)
  (define centrex 300)
  (define centrey 300)
  (define boardsize 5)
  (define myboard (make-board boardsize))
  
  (define first_player 'user)
  (define game 'vscomp)
  (define comp 1)
  (define user -1)
  (define is_lifted? #f)
  (define cor_lifted ())
  
  ((myboard 'set-val) '(4 . 0)  -1)
  ((myboard 'set-val) '(4 . 4)  1)
  ((myboard 'set-val) '(4 . 8)  -1)
  ((myboard 'set-val) '(4 . 12)  1)
  ((myboard 'set-val) '(4 . 16)  -1)
  ((myboard 'set-val) '(4 . 20)  1)
  ((myboard 'set-val) '(0 . 0)  ())
  ((myboard 'set-val) '(2 . 3)  ())
  ((myboard 'set-val) '(2 . 9)  ())
  
  (define replace_board (copy-board (myboard 'board)))  
  
  
  ;;;;;;;;;;;;;;;;colours;;;;;;;;;;;;;;;;;;;
  
  (define col_white (make-object color% 255 255 255))
  (define col_black (make-object color% 0 0 0))
  
  ;;;;;;;;;;;;;;;;;;;;;;Bitmaps;;;;;;;;;;;;;;;;;;;;;;;;;
  (define hex00 (make-object bitmap% (build-path basepath "images" "board" "hex00.xpm") 'xpm))
  (define hex01 (make-object bitmap% (build-path basepath "images" "board" "hex01.xpm") 'xpm))
  (define hex02 (make-object bitmap% (build-path basepath "images" "board" "hex02.xpm") 'xpm))
  (define hex10 (make-object bitmap% (build-path basepath "images" "board" "hex10.xpm") 'xpm))
  (define hex11 (make-object bitmap% (build-path basepath "images" "board" "hex11.xpm") 'xpm))
  (define hex20 (make-object bitmap% (build-path basepath "images" "board" "hex20.xpm") 'xpm))
  (define hex21 (make-object bitmap% (build-path basepath "images" "board" "hex21.xpm") 'xpm))
  (define hmask (make-object bitmap% (build-path basepath "images" "board" "hmask.xpm") 'xpm))
  
  
  
  ;                                                                  
  ;                                                                  
  ;                                                                  
  ;                                                                  
  ;                                   ;;         ;                   
  ;                                    ;                             
  ;    ;;; ;; ;; ;;;   ;;;;   ;; ;;    ; ;;    ;;;     ;;; ;   ;;;;; 
  ;   ;   ;;   ;;     ;    ;   ;;  ;   ;;  ;     ;    ;   ;;  ;    ; 
  ;   ;    ;   ;       ;;;;;   ;   ;   ;   ;     ;    ;        ;;;;  
  ;   ;    ;   ;      ;    ;   ;   ;   ;   ;     ;    ;            ; 
  ;   ;   ;;   ;      ;   ;;   ;   ;   ;   ;     ;    ;    ;  ;    ; 
  ;    ;;; ;  ;;;;;    ;;; ;;  ;;;;   ;;; ;;;  ;;;;;   ;;;;   ;;;;;  
  ;        ;                   ;                                     
  ;    ;;;;                   ;;;                                    
  ;                                                                  
  ;                                                                  
  
  ;; Make a frame
  (define frame (instantiate frame% ("Hexxagon - Project CS-154") (width 800) (height 650) (style '(no-resize-border))))
  (define horz_pane (new horizontal-pane% (parent frame) (vert-margin 0) (horiz-margin 0) (border 0)))
  
  ;; buffer bitmap;;;;;;;;;;;;;;;;
  (define dc (new bitmap-dc% (bitmap (make-object bitmap% 650 650 #f)))) 
  (send dc set-background col_black)
  (send dc clear)
  
  ;;;;;;;;;;;;;;;;;custom canvas;;;;;;;;;;;;;;;;;;;;;;;;;
  (define my-canvas%
    (class canvas% ; The base class is canvas%
      ;; Declare overrides:
      (override on-event)
      ;; Define overriding method to handle mouse events
      (define on-event (lambda (event) (if (eq? (send event get-event-type) 'left-up) (mouseclick (cons (send event get-x) (send event get-y))) void)))
      ;; Call the superclass initialization (and pass on all init args)
      (super-instantiate ())))
  
  ;; Make the drawing area
  (define canvas (instantiate my-canvas% (horz_pane '(transparent) (lambda (x y) (send y clear) (send y draw-bitmap (send dc get-bitmap) 0 0)) #f #f #t 0 0 500)))
  
  ;; Get the canvas's drawing context
  (define canvas-dc (send canvas get-dc))
  (send canvas-dc set-background col_black)
  
  ;;make notification area
  (define vert_pane (new vertical-pane% (parent horz_pane) (vert-margin 0) (horiz-margin 0) (border 0)))
  
  (define notification-dc (send (new canvas% (parent vert_pane) (paint-callback (lambda (x y) (send y clear))) (min-height 200) (stretchable-height #f)) get-dc))
  (send notification-dc set-background col_black)
  
  (let* ((defaultfont (make-object font%))
         (newfont (make-object font% 25 (send defaultfont get-family) 'normal 'bold #f 'default #t)))
    (send notification-dc set-font newfont))
  
  (send notification-dc set-text-foreground col_white) 
  
  (send frame show #t)
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;draw point;;;;;;;;;;;;;;;;;;
  
  (define (drawpoint r theta val border_level)
    (define xy (rtheta->xy r theta))
    (define x (- (car xy) (/ d (sqrt 3))))
    (define y (- (cdr xy) (* 1.2 d)))
    
    (cond
      ((eq? val 1)
       (if (= border_level (- 1))
           (send dc draw-bitmap hex11 x y 'xor  col_black hmask)
           (send dc draw-bitmap hex10 x y 'xor  col_black hmask)))
      ((eq? val (- 1))
       (if (= border_level (- 1))
           (send dc draw-bitmap hex21 x y 'xor  col_black hmask)
           (send dc draw-bitmap hex20 x y 'xor  col_black hmask)))
      ((eq? val 0) 
       (cond
         ((= border_level 2) (send dc draw-bitmap hex02 x y 'xor  col_black hmask))
         ((= border_level 1) (send dc draw-bitmap hex01 x y 'xor  col_black hmask))
         (else (send dc draw-bitmap hex00 x y 'xor  col_black hmask))))))
  
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;  draw array  ;;;;;;;;;;;;;;;;;;;;;
  (define (drawlist points_vector ringno)
    (define points_list (vector->list points_vector))
    
    (define (helper x y)
      (let 
          ((r (car x))
           (theta (cdr x)))
        (drawpoint r theta ((myboard 'get-val) (cons r theta)) ringno)))
    
    (foldr helper void points_list))
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;; draw board;;;;;;;;;;;;;;;;;;;;;;;;
  (define (draw m)
    (define board (m 'board))
    
    (define (countdraw r theta)
      (if (>= r (- (vector-length board) 2)) void
          (if (>= theta (vector-length (vector-ref board r))) (countdraw (+ r 1) 0)
              (begin
                (drawpoint r theta (vector-ref (vector-ref board r) theta) 0)
                (countdraw r (+ theta 1))))))
    
    (send dc clear)
    (countdraw 0 0)
    
    (send canvas-dc draw-bitmap (send dc get-bitmap) 0 0))
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;show-game-end-msg;;;;;;;;;;;;;;
  
  (define (show-game-end msgno)
    
    (let*
        ((bmp (make-object bitmap% (build-path basepath "images" (string-append "msg" (number->string msgno) ".xpm" )) 'xpm))
         (msg-frame (new frame% (label "End Message") (width 250) (height 150) (x (+ 250 (send frame get-x))) (y (+ 200 (send frame get-y))) (style '(float no-caption))))
         (my-canvas%
          (class canvas% ; The base class is canvas%
            ;; Declare overrides:
            (override on-event)
            ;; Define overriding method to handle mouse events
            (define on-event (lambda (event) 
                               (if (eq? (send event get-event-type) 'left-up)
                                   (begin
                                     (send msg-frame show #f)
                                     (send frame enable #t)
                                     (send frame show #t) 
                                     (new-game))
                                   void)))
            ;; Call the superclass initialization (and pass on all init args)
            (super-instantiate ()))))
      (instantiate my-canvas% (msg-frame '(transparent) (lambda (x y) (send y clear) (send y draw-bitmap bmp 0 0))))
      (send frame enable #f)
      (send msg-frame show #t)))
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;show-working;;;;;;;;;;;;;;;;;;;;;;;;;
  
  (define (show-working x)
    (if x (send notification-dc draw-text "Thinking" 0 50 #f 0 0) (send notification-dc clear)))
  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;end of graphics;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  
  
  
  ;                                                                                                  
  ;                                                                                                  
  ;                                                                                                  
  ;                                                                                                  
  ;                                                                                             ;;   
  ;                                                                    ;                         ;   
  ;  ;; ;  ;   ;;;;   ;; ;;   ;;  ;;           ;;; ;   ;;;;   ;; ;;   ;;;;;   ;; ;;;   ;;;;      ;   
  ;   ;; ;; ; ;    ;   ;;  ;   ;   ;          ;   ;;  ;    ;   ;;  ;   ;       ;;     ;    ;     ;   
  ;   ;  ;  ; ;;;;;;   ;   ;   ;   ;  ;;;;;;  ;       ;    ;   ;   ;   ;       ;      ;    ;     ;   
  ;   ;  ;  ; ;        ;   ;   ;   ;          ;       ;    ;   ;   ;   ;       ;      ;    ;     ;   
  ;   ;  ;  ; ;        ;   ;   ;  ;;          ;    ;  ;    ;   ;   ;   ;   ;   ;      ;    ;     ;   
  ;  ;;; ;; ;  ;;;;;  ;;; ;;;   ;; ;;          ;;;;    ;;;;   ;;; ;;;   ;;;   ;;;;;    ;;;;    ;;;;; 
  ;                                                                                                  
  ;                                                                                                  
  ;                                                                                                  
  ;                                                                                                  
  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;menu-control;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  (define (menu-control)
    
    (define buttons ())
    (define current_y 50)
    
    ;buffer bitmap;;;;;;;;;;;;;;;;
    (define menu_bar_dc (new bitmap-dc% (bitmap (make-object bitmap% 200 650 #f)))) 
    
    ;;;;;;;;;;;;;;;;;custon canvas;;;;;;;;;;;;;;;;;;;;;;;;;
    (define my-canvas%
      (class canvas% ; The base class is canvas%
        ;; Declare overrides:
        (override on-event)
        ;; Define overriding method to handle mouse events
        (define on-event (lambda (event) (if (eq? (send event get-event-type) 'left-up) 
                                             (mouseclick 'mouse_up (send event get-x) (send event get-y))
                                             (if (eq? (send event get-event-type) 'left-down) 
                                                 (mouseclick 'mouse_down (send event get-x) (send event get-y)) 
                                                 void))))
        ;; Call the superclass initialization (and pass on all init args)
        (super-instantiate ())))
    
    ;; Make the drawing area
    (define canvas (instantiate my-canvas% (vert_pane '(transparent) (lambda (x y) (send y clear) (send y draw-bitmap (send menu_bar_dc get-bitmap) 0 0)))))
    
    ;; Get the canvas's drawing context
    (define canvas-dc (send canvas get-dc))
    
    (define (add-button image_base f)
      (define y_offset 20)
      
      (define (make-button left top)
        (define image_up (make-object bitmap% (build-path basepath "images" "buttons" (string-append image_base "1.xpm" )) 'xpm))
        (define image_down (make-object bitmap% (build-path basepath "images" "buttons" (string-append image_base "2.xpm" )) 'xpm))
        (define bottom (+ top (send image_up get-height)))
        (define right (+ left (send image_up get-width)))
        (define isdown? #f)
        
        (define (is-inside? x y) (and (> x left) (< x right) (> y top) (< y bottom)))
        (define (me msg x y)
          (if (eq? msg 'mouse_up)
              (if isdown? 
                  (begin
                    (set! isdown? #f)
                    (send menu_bar_dc draw-bitmap image_up left top)
                    (if (is-inside? x y) (f) void))
                  void)
              (if (is-inside? x y)
                  (begin
                    (set! isdown? #t)
                    (send menu_bar_dc draw-bitmap image_down left top))
                  void)))
        (send menu_bar_dc draw-bitmap image_up left top)
        (set! current_y (+ bottom y_offset))
        me)
      
      (set! buttons (cons (make-button 25 current_y) buttons))
      'ok)
    
    
    (define (mouseclick msg x y)
      (define (loop l)
        (if (null? l) void
            (begin
              ((car l) msg x y)
              (loop (cdr l)))))
      (loop buttons)
      (send canvas-dc draw-bitmap (send menu_bar_dc get-bitmap) 0 0))
    
    (define (get-files)
      (define (loop l)
        (if (null? l)
            ()
            (cons (path->string (car l)) (loop (cdr l)))))
      (loop (directory-list (build-path basepath "saved"))))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;save-menu;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (save-menu)
      (define (dosave b e)
        (with-handlers ((exn:fail:filesystem:exists? (lambda (exn) (message-box "Error" "File already exists" save-frame '(ok stop))))
                        (exn:fail? (lambda (exn) (message-box "Error" "File Handling error" save-frame '(ok stop)))))
          (let*
              ((path (build-path basepath "saved" (send file-name get-value)))
               (out-port (open-output-file path)))
            (write (myboard 'board) out-port)
            (close-output-port out-port) 
            (send save-frame show #f))))
      
      (define (dodelete b e)
        (with-handlers ((exn:fail? (lambda (exn) (message-box "Error" "File Handling error" save-frame '(ok stop)))))
          (let*
              ((path (build-path basepath "saved" (list-ref list_of_files (car (send file-list get-selections))))))
            (delete-file path)
            (set! list_of_files (get-files))
            (send file-list set list_of_files))))
      
      
      (define list_of_files (get-files))
      (define save-frame (new dialog% (label "Save Game") (parent frame) (x (+ 300 (send frame get-x))) (y (+ 250 (send frame get-y)))))
      (define file-name (new text-field% (label "Save As") (parent save-frame) (min-width 70)))
      (define file-list (new list-box% (label #f) (choices list_of_files) (parent save-frame)))
      (define horz_pane (new horizontal-pane% (parent save-frame)))
      (define save (new button% (label "Save") (parent horz_pane) (callback dosave)))
      (define cancel (new button% (label "Cancel") (parent horz_pane) (callback (lambda (b e) (send save-frame show #f)))))
      (define delete (new button% (label "Delete") (parent horz_pane) (callback dodelete)))
      
      (send save-frame show #t))
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;load-menu;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (load-menu)
      (define (doload b e)
        (with-handlers ((exn:fail? (lambda (exn) (message-box "Error" "File Handling error" load-frame '(ok stop)))))
          (let*
              ((path (build-path basepath "saved" (list-ref list_of_files (car (send file-list get-selections)))))
               (in-port (open-input-file path)))
            ((myboard 'replace-board) (read in-port))
            (draw myboard)
            (close-input-port in-port)
            (send load-frame show #f))))
      
      (define (dodelete b e)
        (with-handlers ((exn:fail? (lambda (exn) (message-box "Error" "File Handling error" load-frame '(ok stop)))))
          (let*
              ((path (build-path basepath "saved" (list-ref list_of_files (car (send file-list get-selections))))))
            (delete-file path)
            (set! list_of_files (get-files))
            (send file-list set list_of_files))))
      
      
      (define list_of_files (get-files))
      (define load-frame (new dialog% (label "Load Game") (parent frame) (x (+ 300 (send frame get-x))) (y (+ 250 (send frame get-y)))))
      (define file-list (new list-box% (label #f) (choices list_of_files) (parent load-frame)))
      (define horz_pane (new horizontal-pane% (parent load-frame)))
      (define load (new button% (label "Load") (parent horz_pane) (callback doload)))
      (define cancel (new button% (label "Cancel") (parent horz_pane) (callback (lambda (b e) (send load-frame show #f)))))
      (define delete (new button% (label "Delete") (parent horz_pane) (callback dodelete)))
      
      (send load-frame show #t))
    
    ;;;;;;;;;;;;;;;;;;;;;;;option-menu;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (define (option-menu)
      
      (define (dook b e)
        (set! ply (+ 1 (send ply-choose get-selection)))
        (set! first_player (if (= (send first-option get-selection) 0) 'comp 'user))
        (send option-frame show #f))
      
      (define (doedit b e)
        (send option-frame show #f)
        (edit-board))
      
      (define option-frame (new dialog% (label "Options") (parent frame) (x (+ 300 (send frame get-x))) (y (+ 250 (send frame get-y)))))
      (define first-option (new radio-box% (label "First Player") (choices '("Computer" "User")) (parent option-frame)))
      (define ply-choose (new choice% (label "Level") (choices '("1" "2" "3" "4" "5" "6")) (parent option-frame))) 
      (define horz_pane (new horizontal-pane% (parent option-frame)))
      (define ok (new button% (label "OK") (parent horz_pane) (callback dook)))
      (define cancel (new button% (label "Cancel") (parent horz_pane) (callback (lambda (b e) (send option-frame show #f)))))
      (define edit (new button% (label "Edit Board...") (parent horz_pane) (callback doedit)))
      
      (send ply-choose set-selection (- ply 1))
      (send first-option set-selection (if (eq? first_player 'comp) 0 1))
      (send option-frame show #t))
    
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    
    (send menu_bar_dc set-background col_black)
    (send menu_bar_dc clear)
    (send canvas-dc set-background col_black)
    
    
    (add-button "new" new-game)
    (add-button "load" load-menu)
    (add-button "save" save-menu)
    (add-button "options" option-menu)
    (add-button "exit" (lambda () (send frame show #f)))
    (send canvas-dc draw-bitmap (send menu_bar_dc get-bitmap) 0 0))
  
  (menu-control)
  
  
  ;                                                                                  
  ;                                                                                  
  ;                                                                                  
  ;                                                                                  
  ;               ;;     ;                   ;;                                   ;; 
  ;                ;           ;              ;                                    ; 
  ;    ;;;;    ;;; ;   ;;;    ;;;;;           ; ;;;    ;;;;    ;;;;   ;; ;;;   ;;; ; 
  ;   ;    ;  ;   ;;     ;     ;              ;;   ;  ;    ;  ;    ;   ;;     ;   ;; 
  ;   ;;;;;;  ;    ;     ;     ;      ;;;;;;  ;    ;  ;    ;   ;;;;;   ;      ;    ; 
  ;   ;       ;    ;     ;     ;              ;    ;  ;    ;  ;    ;   ;      ;    ; 
  ;   ;       ;   ;;     ;     ;   ;          ;;   ;  ;    ;  ;   ;;   ;      ;   ;; 
  ;    ;;;;;   ;;; ;;  ;;;;;    ;;;          ;; ;;;    ;;;;    ;;; ;; ;;;;;    ;;; ;;
  ;                                                                                  
  ;                                                                                  
  ;                                                                                  
  ;                                                                                  ;
  
  (define (edit-board)
    (define editor-frame (new frame% (label "Edit Board") (width 800) (height 650) (x (send frame get-x)) (y (send frame get-y)) (style '(no-resize-border))))
    (define horz_pane (new horizontal-pane% (parent editor-frame) (vert-margin 0) (horiz-margin 0) (border 0)))
    (define menu-dc (new bitmap-dc% (bitmap (make-object bitmap% 200 650 #f))))
    (define selected 0)
    (define original_board (copy-board (myboard 'board)))
    
    (define (mouseclick xy)
      (define r-theta (xy->rtheta xy))
      (if (not (eq? r-theta 'out-of-board))
          (begin
            ((myboard 'set-val) r-theta selected)
            (draw myboard)
            (send (send editor-canvas get-dc) draw-bitmap (send dc get-bitmap) 0 0))
          void))
    
    (define (dook b e)
      (set! replace_board (copy-board (myboard 'board)))
      (send editor-frame show #f)
      ((myboard 'replace-board) replace_board)
      
      (send frame show #t)
      
      (draw myboard)
      (if (eq? first_player 'comp) (comp-play) void))
    
    
    (define (docancel b e)
      (send editor-frame show #f)
      ((myboard 'replace-board) original_board)
      
      (send frame show #t)
      
      (draw myboard))
    
    (define my-canvas%
      
      (class canvas% ; The base class is canvas%
        ;; Declare overrides:
        (override on-event)
        ;; Define overriding method to handle mouse events
        (define on-event (lambda (event) (if (eq? (send event get-event-type) 'left-up) (mouseclick (cons (send event get-x) (send event get-y))) void)))
        ;; Call the superclass initialization (and pass on all init args)
        (super-instantiate ())))
    
    ;; Make the drawing area
    (define editor-canvas (instantiate my-canvas% (horz_pane '(transparent) (lambda (x y) (send y clear) (send y draw-bitmap (send dc get-bitmap) 0 0)) #f #f #t 0 0 500)))
    
    (define menubar (new vertical-pane% (parent horz_pane)))
    (define selectedimage ())
    
    
    (new button% (label hex00) (parent menubar) (callback (lambda (b e) (set! selected 0) (send selectedimage set-label hex00))))
    (new button% (label hex10) (parent menubar) (callback (lambda (b e) (set! selected 1) (send selectedimage set-label hex10))))
    (new button% (label hex20) (parent menubar) (callback (lambda (b e) (set! selected -1) (send selectedimage set-label hex20))))
    (new button% (label hmask) (parent menubar) (callback (lambda (b e) (set! selected ()) (send selectedimage set-label hmask))))
    (new vertical-pane% (parent menubar) (min-height 100) (stretchable-height #f))
    
    (new button% (label "OK") (parent menubar) (callback dook))
    (new button% (label "Cancel") (parent menubar) (callback docancel))
    
    (new vertical-pane% (parent menubar) (min-height 100) (stretchable-height #f))
    (new message% (label "Selected") (parent menubar))
    (set! selectedimage (new message% (label hex00) (parent menubar)))
    
    
    (send frame show #f)
    
    ((myboard 'replace-board) replace_board)
    (send editor-frame show #t)
    (draw myboard)
    (send (send editor-canvas get-dc) draw-bitmap (send dc get-bitmap) 0 0))
  
  
  (draw myboard)
  
  (sleep/yield 2)
  (send splash-frame show #f)
  
  )